import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleLinear, format, extent } from 'd3';
import { useData } from './useData';
import { AxisLeft } from './AxisLeft';
import { AxisBottom } from './AxisBottom';
import { Marks } from './Marks';
import { Dropdown } from './Dropdown';

const width = window.innerWidth;
const menuHeight = 45;
const height = window.innerHeight - menuHeight;
const margin = {
  top: 20,
  right: 30,
  bottom: 65,
  left: 90,
};

const xAxisLabelOffset = 50;
const yAxisLabelOffset = 45;

const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;

const App = () => {
  const data = useData();
  
  const initialXAttribute = 'petal_length';
  const [xAttribute, setXAttribute] = useState(initialXAttribute);
  const xValue = d => d[xAttribute];
  
  const initialYAttribute = 'sepal_width';
  const [yAttribute, setYAttribute] = useState(initialYAttribute);
  const yValue = (d) => d[yAttribute];
  
  if (!data) {
    return <pre>Loading...</pre>;
  }
  
  const options = [];
  let row = {};
  data.columns.forEach(d => { 
    if(d==="species"){return;};
    row = {value: d, label: d.split("_").map(d => d[0].toUpperCase() + d.slice(1)).join(" ")};
		options.push(row);
  });

  const xAxisLabel= options.filter(d => d.value === xAttribute)[0].label;
  const yAxisLabel = options.filter(d => d.value === yAttribute)[0].label;
  const yAxisTickFormat = format('.2f');
  
  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]);

  return (
    <>
      <label for="x-select">X:</label>
      <Dropdown
        options={options}
        id="x-select"
        selectedValue={xAttribute}
        onSelectedValueChange={setXAttribute}
      />
      
      <label for="y-select">Y:</label>
      <Dropdown
        options={options}
        id="y-select"
        selectedValue={yAttribute}
        onSelectedValueChange={setYAttribute}
      />
      
      <svg width={width} height={height}>
        <g
          transform={`translate(${margin.left}, ${margin.top})`}
        >
          <AxisLeft
            yScale={yScale}
            tickFormat={yAxisTickFormat}
            innerWidth={innerWidth}
            tickOffset={5}
          />
          <text
            className="axis-label"
            textAnchor="middle"
            x={-yAxisLabelOffset}
            y={innerHeight / 2}
            style={{
              transformOrigin: `${-yAxisLabelOffset}px ${
                innerHeight / 2
              }px`,
            }}
            transform="rotate(-90)"
          >
            {yAxisLabel}
          </text>

          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            tickOffset={5}
          />
          <text
            className="axis-label"
            x={innerWidth / 2}
            y={innerHeight + xAxisLabelOffset}
            textAnchor="middle"
          >
            {xAxisLabel}
          </text>

          <Marks
            data={data}
            xScale={xScale}
            yScale={yScale}
            xValue={xValue}
            yValue={yValue}
            tooltipFormat={yAxisTickFormat}
            circleRadius={7}
          />
        </g>
      </svg>
    </>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
